/*
 * Copyright (C) 2017-2022 Dominique Dhoutaut, Thierry Arrabal, Eugen Dedu.
 *
 * This file is part of BitSimulator.
 *
 * BitSimulator is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * BitSimulator is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BitSimulator.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef AGENTS_ROUTING_AGENT_H_
#define AGENTS_ROUTING_AGENT_H_

#include <iostream>
#include "../node.h"


//==============================================================================
//
//          RoutingAgent  (class)
//
//==============================================================================

class RoutingAgent {
protected:
  Node *hostNode;
  static int aliveAgents;
  static int forwardedDataPackets;
  simulationTime_t getBackoffDynamic (Node *hostNode, PacketPtr packet, float multiplier, mt19937_64 *rng);
  simulationTime_t getBackoffDynamicRing (Node *hostNode, PacketPtr packet, float multiplier, mt19937_64 *rng);
public:
  static int anchor1id, anchor2id, anchor3id;
  static void initializeAgent() { cout << "Initializing RoutingAgent" << endl; }
  RoutingAgent(Node *_hostNode);
  virtual ~RoutingAgent();

  virtual void receivePacketFromNetwork(PacketPtr _packet) = 0;
  virtual void receivePacketFromApplication(PacketPtr _packet);
  virtual void processAckTimedOut(PacketPtr _packet);
  virtual void processBackoffSending(int packetId);
  virtual void processSLRBackoffSending(int packetId);

  static void slrOpenFile (ofstream &file);
  static void readCommRangeSetupAttribute ();
  static void readAnchorAttributes ();
  static void readRangeAttributes ();
  static void readProbaAttribute ();
  static int DeltaA(int dstX, int dstY, int dstZ,int srcX,int srcY,int srcZ,int X,int Y, int Z );
  static int DeltaB(int dstX, int dstY, int dstZ,int srcX,int srcY,int srcZ,int X,int Y, int Z );
};

#endif /* AGENTS_ROUTING_AGENT_H_ */
